/*
 * 21:22 - 28/06/2003
 *
 * WinampWide.cpp - A tiny application which allows to manipulate Winamp
 * Copyright (C) 2003 Romain Guy
 * romain.guy@jext.org
 * www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//////////////////////////////////////////////////////////////////
// INCLUDES
//////////////////////////////////////////////////////////////////

#include "WinampWide.h"
#include "WinampWideDll.h"
#include "WinampWideTypes.h"
#include "resource.h"

#include <windows.h>
#include <stdio.h>

//////////////////////////////////////////////////////////////////
// MESSAGES HANDLING
//////////////////////////////////////////////////////////////////

INT_PTR CALLBACK DialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  int wmId;
  int wmEvent;

  POINT pt;
  HMENU hMenu;

  switch (message) 
  {
  case WINAMPWIDE_TRAY_MESSAGE:
    switch(lParam)
    {
    case WM_RBUTTONDOWN:
    case WM_CONTEXTMENU:
      GetCursorPos(&pt);
      hMenu = CreatePopupMenu();
      if (hMenu)
      {
        InsertMenu(hMenu, -1, MF_BYPOSITION, WINAMPWIDE_EXIT_MESSAGE, "Exit");
        SetForegroundWindow(hWnd);
        TrackPopupMenu(hMenu, TPM_BOTTOMALIGN, pt.x, pt.y, 0, hWnd, NULL);
        DestroyMenu(hMenu);
      }
    }
    break;
  case WM_COMMAND:
    wmId    = LOWORD(wParam);
    wmEvent = HIWORD(wParam);

    switch (wmId)
    {
		case WINAMPWIDE_EXIT_MESSAGE:
      PostThreadMessage(GetCurrentThreadId(), WM_DESTROY, 0, 0);
      break;
    }
    return 1;
  case WM_INITDIALOG:
    return 1;
  }

  return 0;
}

//////////////////////////////////////////////////////////////////
// WINDOWS ENTRY POINT
//////////////////////////////////////////////////////////////////

int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPTSTR    lpCmdLine,
                   int       nCmdShow)
{
  MSG msg;
  NOTIFYICONDATA nid;
  HWND hWnd;

  // hidden dialog to receive and handle windows messages
  hWnd = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_WINAMPWIDE), NULL, (DLGPROC) DialogProc);

  // adding icon in the taskbar notification area
  ZeroMemory(&nid, sizeof(NOTIFYICONDATA));
  nid.cbSize = sizeof(NOTIFYICONDATA);
  nid.hWnd   = hWnd;
  nid.uID    = IDI_TRAY_ICON;
  nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
  nid.hIcon  = (HICON) LoadImage(hInstance,
                                 MAKEINTRESOURCE(IDI_TRAY_ICON),
                                 IMAGE_ICON,
                                 GetSystemMetrics(SM_CXSMICON),
                                 GetSystemMetrics(SM_CYSMICON),
                                 LR_DEFAULTCOLOR);
  nid.uCallbackMessage = WINAMPWIDE_TRAY_MESSAGE;
  lstrcpyn(nid.szTip, "WinampWide\nCode(r) 2004", sizeof(nid.szTip) / sizeof(TCHAR));
  Shell_NotifyIcon(NIM_ADD, &nid);
  if (nid.hIcon && DestroyIcon(nid.hIcon))
    nid.hIcon = NULL;

  // winamp wide initialization
  WinampWide_InstallHook(GetCurrentThreadId());

  while (GetMessage(&msg, NULL, 0, 0)) 
  {
    if (!TranslateAccelerator(hWnd, NULL, &msg))
    { 
      TranslateMessage(&msg); 
      DispatchMessage(&msg); 
    }

    if (msg.message == WM_DESTROY)
      break;
  }

  WinampWide_UnInstallHook();
  Shell_NotifyIcon(NIM_DELETE, &nid);
  DestroyWindow(hWnd);

  return 0;
}

// end of WinampWide.cpp