/*
 * 21:22 - 28/06/2003
 *
 * WinampWideDll.cpp - A tiny application which allows to manipulate Winamp
 * Copyright (C) 2003 Romain Guy
 * romain.guy@jext.org
 * www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//////////////////////////////////////////////////////////////////
// INCLUDES
//////////////////////////////////////////////////////////////////

#include "WinampWideDll.h"
#include "WinampWideTypes.h"

#include <windows.h>
#include <stdio.h>

//////////////////////////////////////////////////////////////////
// DEFINES
//////////////////////////////////////////////////////////////////

#define WA_PREVIOUS              40044      // play previous song
#define WA_PLAY                  40045      // play song
#define WA_PAUSE                 40046      // pause song
#define WA_STOP                  40047      // stop playing
#define WA_NEXT                  40048      // play next song

#define WA_SHUTDOWN              40001      // shutdown winamp

//////////////////////////////////////////////////////////////////
// GLOBALS
//////////////////////////////////////////////////////////////////

// Dev-C++/mingw/gcc win32
#ifdef _DEVCPP
HHOOK        g_hHook     __attribute__((section ("shared"), shared)) = NULL;
HINSTANCE    g_hInstance __attribute__((section ("shared"), shared)) = NULL;
DWORD        g_threadId  __attribute__((section ("shared"), shared)) = 0;
// Visual C++ 6.0
#else
#pragma comment(linker, "/SECTION:.SHARDATA,RWS")
#pragma data_seg(".SHARDATA")
HHOOK        g_hHook     = NULL;           // handle to the hook
HINSTANCE    g_hInstance = NULL;           // handle to the current DLL instance
DWORD        g_threadId  = 0;              // handle to application thread ID
#pragma data_seg()
#endif

pWawKeyCodes g_keysConf  = NULL;           // handle to keys configuration

//////////////////////////////////////////////////////////////////
// HOOKS
//////////////////////////////////////////////////////////////////

// keyboard hook, system wide

LRESULT CALLBACK WinampWide_KeyboardHook(int nCode, WPARAM wParam, LPARAM lParam)
{
  if (nCode == HC_ACTION)
  {
    // Ctrl+Shift+[A key]
    // if the key wasn't down before this call, we skip the message
    if (GetAsyncKeyState(VK_CONTROL) && GetAsyncKeyState(VK_SHIFT) && !(lParam & 0x040000000))
    {
      UINT message    = 0;
      HWND hwndWinamp = FindWindow("Winamp v1.x", NULL);
      UINT scanCode   = MapVirtualKeyEx(wParam, 0, GetKeyboardLayout(0));

      if (g_keysConf == NULL)
        g_keysConf = WinampWide_SetupKeys();
      
      if (scanCode == g_keysConf->previous)
        message = WA_PREVIOUS;
      else if (scanCode == g_keysConf->play)
        message = WA_PLAY;
      else if (scanCode == g_keysConf->pause)
        message = WA_PAUSE;
      else if (scanCode == g_keysConf->stop)
        message = WA_STOP;
      else if (scanCode == g_keysConf->next)
        message = WA_NEXT;
      else if (scanCode == g_keysConf->shutdown)
        message = WA_SHUTDOWN;
      else if (scanCode == g_keysConf->exit)
        PostThreadMessage(g_threadId, WM_DESTROY, 0, 0);

      if (message)
        SendMessage(hwndWinamp, WM_COMMAND, message, 0);
    }
  }

  return CallNextHookEx(g_hHook, nCode, wParam, lParam);
}

// installs the hook

HOOKDLL_API BOOL WinampWide_InstallHook(DWORD threadId)
{
  if (g_hHook == NULL)
  {
    g_hHook    = SetWindowsHookEx( WH_KEYBOARD, (HOOKPROC) WinampWide_KeyboardHook,
                                   g_hInstance, 0);
    g_threadId = threadId;
  }

	return (BOOL) g_hHook;
}

// uninstalls the hook

BOOL HOOKDLL_API WinampWide_UnInstallHook()
{
  BOOL resultOk = TRUE;

  if (g_hHook != NULL)
  {
    resultOk = UnhookWindowsHookEx(g_hHook);
    g_hHook = NULL;
  }

  return resultOk;
}

// stores the keys configuration

pWawKeyCodes WinampWide_SetupKeys(void)
{
  int nbChars;
  SHORT vkeyCode;

  LPSTR iniPath;
  LPSTR lastSlash;
  LPSTR buffer;

  pWawKeyCodes keys;

  iniPath = (LPSTR) malloc(MAX_PATH);
  buffer  = (LPSTR) malloc(2);
  keys    = malloc(sizeof(WawKeyCodes));

  // build up the full access path to the ini file
  GetModuleFileName(g_hInstance, iniPath, (DWORD) MAX_PATH);
  lastSlash = strrchr(iniPath, '\\');
  *(++lastSlash) = '\0'; 
  strcat(iniPath, WAW_INI_FILE);
  
  // get key chars
  nbChars  = GetPrivateProfileString("WinampWide", "PREVIOUS", "&",  buffer, 2, iniPath);
  vkeyCode = VkKeyScanEx(buffer[0], GetKeyboardLayout(0));
  keys->previous = MapVirtualKeyEx(vkeyCode, 0, GetKeyboardLayout(0));
  
  nbChars  = GetPrivateProfileString("WinampWide", "PLAY", "",  buffer, 2, iniPath);
  vkeyCode = VkKeyScanEx(buffer[0], GetKeyboardLayout(0));
  keys->play = MapVirtualKeyEx(vkeyCode, 0, GetKeyboardLayout(0));
  
  nbChars  = GetPrivateProfileString("WinampWide", "PAUSE", "\"", buffer, 2, iniPath);
  vkeyCode = VkKeyScanEx(buffer[0], GetKeyboardLayout(0));
  keys->pause = MapVirtualKeyEx(vkeyCode, 0, GetKeyboardLayout(0));
  
  nbChars  = GetPrivateProfileString("WinampWide", "STOP", "'",  buffer, 2, iniPath);
  vkeyCode = VkKeyScanEx(buffer[0], GetKeyboardLayout(0));
  keys->stop = MapVirtualKeyEx(vkeyCode, 0, GetKeyboardLayout(0));
  
  nbChars  = GetPrivateProfileString("WinampWide", "NEXT", "(",  buffer, 2, iniPath);
  vkeyCode = VkKeyScanEx(buffer[0], GetKeyboardLayout(0));
  keys->next = MapVirtualKeyEx(vkeyCode, 0, GetKeyboardLayout(0));
  
  nbChars  = GetPrivateProfileString("WinampWide", "SHUTDOWN", "-",  buffer, 2, iniPath);
  vkeyCode = VkKeyScanEx(buffer[0], GetKeyboardLayout(0));
  keys->shutdown = MapVirtualKeyEx(vkeyCode, 0, GetKeyboardLayout(0));
  
  nbChars  = GetPrivateProfileString("WinampWide", "EXIT", "",  buffer, 2, iniPath);
  vkeyCode = VkKeyScanEx(buffer[0], GetKeyboardLayout(0));
  keys->exit = MapVirtualKeyEx(vkeyCode, 0, GetKeyboardLayout(0));
  
  // cleaning up
  free(iniPath);
  free(buffer);

  return keys;
}

//////////////////////////////////////////////////////////////////
// DLL ENTRY POINT
//////////////////////////////////////////////////////////////////

BOOL APIENTRY DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  g_hInstance = hinstDLL;
  return TRUE;
}

// end of WinampWideDll.cpp